@@ -1,13 +1,13 @@
 /***************************************
-  $Header: /home/amb/procmeter3/gtk/RCS/window.c 1.1 2000/12/16 16:36:35 amb Exp $
+  $Header: /home/amb/procmeter3/gtk/RCS/window.c 1.4 2002/12/07 19:33:51 amb Exp $
 
-  ProcMeter - A system monitoring program for Linux - Version 3.3.
+  ProcMeter - A system monitoring program for Linux - Version 3.4.
 
   X Windows interface.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98,99,2000 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000,01,02 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -37,7 +37,6 @@
 
 static gint SleepCallback(gpointer p);
 static void ResizePaneCallback(GtkWidget *w,GdkEventConfigure *event);
-static void ResizePane(void);
 static gint CloseCallback(GtkWidget *w,GdkEvent *event,gpointer data);
 
 
@@ -45,7 +44,7 @@ static gint CloseCallback(GtkWidget *w,GdkEvent *event,gpointer data);
 GtkWidget *toplevel;
 
 /*+ The pane that contains all of the outputs. +*/
-GtkWidget *pane;
+GtkWidget *pane=NULL;
 
 /*+ If the meters are aligned vertically. +*/
 int vertical=1;
@@ -65,16 +64,16 @@ static int initialising=1;
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Start the X-Windows part.
+  Start the X-Windows & GTK part.
 
   int *argc The number of command line arguments.
 
   char **argv The actual command line arguments.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void StartX(int *argc,char **argv)
+void Start(int *argc,char **argv)
 {
- static char procmeter_version[16]="ProcMeter V" PROCMETER_VERSION;
+ static char procmeter_version[]="ProcMeter V" PROCMETER_VERSION;
  char *string;
  int i;
 
@@ -160,10 +159,10 @@ void StartX(int *argc,char **argv)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Stop the X-Windows part.
+  Stop the X-Windows & GTK part.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void StopX(void)
+void Stop(void)
 {
  DestroyMenus();
 
@@ -177,11 +176,21 @@ void StopX(void)
   time_t until The time to sleep until.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void SleepX(time_t until)
+void Sleep(time_t until)
 {
  struct timeval now;
  int delay;
 
+ /* Before we sleep the first time, resize the window. */
+
+ if(initialising)
+   {
+    Resize();
+    initialising=0;
+   }
+
+ /* Sleep */
+
  gettimeofday(&now,NULL);
 
  delay=1000*(until-now.tv_sec)-now.tv_usec/1000;
@@ -209,173 +218,6 @@ void SleepX(time_t until)
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  Perform the updates.
-
-  time_t now The current time.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void UpdateX(time_t now)
-{
- Module *module;
- Output *output;
- ProcMeterOutput *last=NULL;
-
- for(module=Modules;*module;module++)
-    for(output=(*module)->outputs;*output;output++)
-       if((*output)->output_widget &&
-          (((*output)->output->interval && !(now%(*output)->output->interval)) ||
-          (*output)->first))
-         {
-          if(last!=(*output)->output)
-             if((*module)->Update(now,(*output)->output)==-1)
-                fprintf(stderr,"ProcMeter: Error updating %s.%s\n",(*module)->module->name,(*output)->output->name);
-
-          if((*output)->first)
-             (*output)->first--;
-
-          if(!(*output)->first)
-            {
-             if((*output)->type==PROCMETER_GRAPH)
-               {
-                long value=(*output)->output->graph_value;
-                if(value<0)
-                   value=0;
-                if(value>65535)
-                   value=65535;
-                ProcMeterGraphAddDatum((*output)->output_widget,value);
-               }
-             else if((*output)->type==PROCMETER_TEXT)
-                ProcMeterTextChangeData((*output)->output_widget,(*output)->output->text_value);
-             else if((*output)->type==PROCMETER_BAR)
-               {
-                long value=(*output)->output->graph_value;
-                if(value<0)
-                   value=0;
-                if(value>65535)
-                   value=65535;
-                ProcMeterBarAddDatum((*output)->output_widget,value);
-               }
-            }
-
-          last=(*output)->output;
-         }
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Add the default outputs at startup.
-
-  int argc The number of command line arguments.
-
-  char **argv The command line arguments.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void AddDefaultOutputs(int argc,char **argv)
-{
- Output *outputp=NULL;
- Module *modulep=NULL;
- char *string;
- int arg;
-
- if((string=GetProcMeterRC("startup","order")))
-   {
-    char *s=string;
-
-    while(*s && *s==' ')
-       s++;
-
-    while(*s)
-      {
-       int found=0;
-
-       for(modulep=Modules;*modulep;modulep++)
-         {
-          if(!strncmp((*modulep)->module->name,s,strlen((*modulep)->module->name)) &&
-             s[strlen((*modulep)->module->name)]=='.')
-            {
-             for(outputp=(*modulep)->outputs;*outputp;outputp++)
-                if(!strncmp((*outputp)->output->name,&s[strlen((*modulep)->module->name)+1],strlen((*outputp)->output->name)) &&
-                   (s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]=='-' ||
-                    s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==' ' ||
-                    s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==0))
-                  {
-                   if((s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==' ' ||
-                       s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==0) &&
-                      !(*outputp)->output_widget)
-                      AddRemoveOutput(*outputp);
-                   else if(s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='g' &&
-                           (*outputp)->type==PROCMETER_GRAPH &&
-                           !(*outputp)->output_widget)
-                      AddRemoveOutput(*outputp);
-                   else if(s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='t' &&
-                           (*outputp)->type==PROCMETER_TEXT &&
-                           !(*outputp)->output_widget)
-                      AddRemoveOutput(*outputp);
-                   else if(s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='b' &&
-                           (*outputp)->type==PROCMETER_BAR &&
-                           !(*outputp)->output_widget)
-                      AddRemoveOutput(*outputp);
-                   found=1;
-                  }
-
-             if(found)
-                break;
-            }
-         }
-
-       while(*s && *s!=' ')
-          s++;
-       while(*s && *s==' ')
-          s++;
-      }
-   }
-
- for(arg=1;arg<argc;arg++)
-   {
-    int found=0;
-
-    for(modulep=Modules;*modulep;modulep++)
-       if(!strncmp((*modulep)->module->name,argv[arg],strlen((*modulep)->module->name)) &&
-          argv[arg][strlen((*modulep)->module->name)]=='.')
-         {
-          for(outputp=(*modulep)->outputs;*outputp;outputp++)
-             if(!strncmp((*outputp)->output->name,&argv[arg][strlen((*modulep)->module->name)+1],strlen((*outputp)->output->name)) &&
-                (argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]=='-' ||
-                 argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==0))
-               {
-                if(!argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1] &&
-                   !(*outputp)->output_widget)
-                   AddRemoveOutput(*outputp);
-                else if(argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='g' &&
-                        (*outputp)->type==PROCMETER_GRAPH &&
-                        !(*outputp)->output_widget)
-                   AddRemoveOutput(*outputp);
-                else if(argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='t' &&
-                        (*outputp)->type==PROCMETER_TEXT &&
-                        !(*outputp)->output_widget)
-                   AddRemoveOutput(*outputp);
-                else if(argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='b' &&
-                        (*outputp)->type==PROCMETER_BAR &&
-                        !(*outputp)->output_widget)
-                   AddRemoveOutput(*outputp);
-                found=1;
-               }
-
-          if(found)
-             break;
-         }
-
-    if(!*modulep)
-       fprintf(stderr,"ProcMeter: Unrecognised output '%s'\n",argv[arg]);
-   }
-
- initialising=0;
-
- ResizePane();
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Add or remove an output
 
@@ -402,7 +244,7 @@ void AddRemoveOutput(Output output)
  else
    {
     GtkWidget *w=NULL;
-    char *string,str[16];
+    char *string,str[PROCMETER_NAME_LEN+1];
     Output *outputp=NULL;
     Module *modulep,module=NULL;
 
@@ -535,7 +377,49 @@ void AddRemoveOutput(Output output)
     ndisplayed++;
    }
 
- ResizePane();
+ Resize();
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Update a graph output.
+
+  Output output The output to update.
+
+  short value The new value.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void UpdateGraph(Output output,short value)
+{
+ ProcMeterGraphAddDatum(output->output_widget,value);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Update a text output.
+
+  Output output The output to update.
+
+  char *value The new value.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void UpdateText(Output output,char *value)
+{
+ ProcMeterTextChangeData(output->output_widget,value);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Update a bar output.
+
+  Output output The output to update.
+
+  short value The new value.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void UpdateBar(Output output,short value)
+{
+ ProcMeterBarAddDatum(output->output_widget,value);
 }
 
 
@@ -566,7 +450,7 @@ void MoveOutput(Output output1,Output output2,int direction)
 
  gtk_box_reorder_child(GTK_BOX(pane),GTK_WIDGET(output1->output_widget),i2);
 
- ResizePane();
+ Resize();
 
  if(direction==1 && i2>i1)
    {
@@ -595,44 +479,11 @@ void MoveOutput(Output output1,Output output2,int direction)
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  The function called by the timeout to terminate the sleep.
-
-  gint SleepCallback Returns true to repeat the timer.
-
-  gpointer p Not used.
-
-  This function is only ever called from the gtk event loop.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static gint SleepCallback(gpointer p)
-{
- sleeping=0;
- return(FALSE);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  A callback that is activated by a resize event on the parent pane.
-
-  GtkWidget *w The widget that caused the callback.
-
-  GdkEventConfigure *event Not used.
-
-  This function is only ever called from the gtk event loop.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static void ResizePaneCallback(GtkWidget *w,GdkEventConfigure *event)
-{
- ResizePane();
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Resize the pane.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void ResizePane(void)
+void Resize(void)
 {
  gushort psize,size;
  gint width,height;
@@ -726,6 +577,39 @@ static void ResizePane(void)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  The function called by the timeout to terminate the sleep.
+
+  gint SleepCallback Returns true to repeat the timer.
+
+  gpointer p Not used.
+
+  This function is only ever called from the gtk event loop.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static gint SleepCallback(gpointer p)
+{
+ sleeping=0;
+ return(FALSE);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  A callback that is activated by a resize event on the parent pane.
+
+  GtkWidget *w The widget that caused the callback.
+
+  GdkEventConfigure *event Not used.
+
+  This function is only ever called from the gtk event loop.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void ResizePaneCallback(GtkWidget *w,GdkEventConfigure *event)
+{
+ Resize();
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   A callback that is activated by a close window event on the toplevel window.
 
