@@ -1,13 +1,13 @@
 /***************************************
-  $Header: /home/amb/procmeter3/modules/RCS/stat-intr.c 1.4 2000/12/13 17:32:49 amb Exp $
+  $Header: /home/amb/procmeter3/modules/RCS/stat-intr.c 1.9 2004/04/03 16:06:12 amb Exp $
 
-  ProcMeter - A system monitoring program for Linux - Version 3.2b.
+  ProcMeter - A system monitoring program for Linux - Version 3.4b.
 
   Interrupt statistics source file.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1998,99 Andrew M. Bishop
+  This file Copyright 1998,99,2002,03,04 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -16,6 +16,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "procmeter.h"
 
@@ -29,27 +30,27 @@
 /*+ The total statistics +*/
 ProcMeterOutput _output=
 {
- /* char  name[16];         */ "Interrupts",
- /* char *description;      */ "The total number of hardware interrupts per second.",
- /* char  type;             */ PROCMETER_GRAPH|PROCMETER_TEXT|PROCMETER_BAR,
- /* short interval;         */ 1,
- /* char  text_value[16];   */ "0 /s",
- /* long  graph_value;      */ 0,
- /* short graph_scale;      */ 100,
- /* char  graph_units[8];   */ "(%d/s)"
+ /* char  name[];          */ "Interrupts",
+ /* char *description;     */ "The total number of hardware interrupts per second.",
+ /* char  type;            */ PROCMETER_GRAPH|PROCMETER_TEXT|PROCMETER_BAR,
+ /* short interval;        */ 1,
+ /* char  text_value[];    */ "0 /s",
+ /* long  graph_value;     */ 0,
+ /* short graph_scale;     */ 100,
+ /* char  graph_units[];   */ "(%d/s)"
 };
 
 /*+ The per interrupt statistics +*/
 ProcMeterOutput _intr_output=
 {
- /* char  name[16];         */ "Interrupt%d",
- /* char *description;      */ "The number of hardware interrupts number %d (%s) per second.",
- /* char  type;             */ PROCMETER_GRAPH|PROCMETER_TEXT|PROCMETER_BAR,
- /* short interval;         */ 1,
- /* char  text_value[16];   */ "0 /s",
- /* long  graph_value;      */ 0,
- /* short graph_scale;      */ 100,
- /* char  graph_units[8];   */ "(%d/s)"
+ /* char  name[];          */ "Interrupt%d",
+ /* char *description;     */ "The number of hardware interrupts number %d (%s) per second.",
+ /* char  type;            */ PROCMETER_GRAPH|PROCMETER_TEXT|PROCMETER_BAR,
+ /* short interval;        */ 1,
+ /* char  text_value[];    */ "0 /s",
+ /* long  graph_value;     */ 0,
+ /* short graph_scale;     */ 100,
+ /* char  graph_units[];   */ "(%d/s)"
 };
 
 /*+ The extra outputs with multiple interrupts +*/
@@ -61,8 +62,8 @@ ProcMeterOutput *outputs[N_INTR+1+1];
 /*+ The module. +*/
 ProcMeterModule module=
 {
- /* char name[16];          */ "Stat-Intr",
- /* char *description;      */ "Interrupt statistics. [From /proc/stat]",
+ /* char name[];           */ "Stat-Intr",
+ /* char *description;     */ "Interrupt statistics. [From /proc/stat]",
 };
 
 /*+ The number of interrupts. +*/
@@ -94,7 +95,7 @@ ProcMeterModule *Load(void)
 ProcMeterOutput **Initialise(char *options)
 {
  FILE *f;
- char line[BUFFLEN+1];
+ char line[BUFFLEN+1],*l;
  int n=0;
 
  outputs[0]=NULL;
@@ -109,26 +110,25 @@ ProcMeterOutput **Initialise(char *options)
     fprintf(stderr,"ProcMeter(%s): Could not open '/proc/stat'.\n",__FILE__);
  else
    {
-    if(!fgets(line,BUFFLEN,f)) /* cpu */
+    l=fgets(line,256,f); /* cpu */
+    if(!l)
        fprintf(stderr,"ProcMeter(%s): Could not read '/proc/stat'.\n",__FILE__);
     else
       {
+       unsigned long intr;
        int i,p,pp;
 
-       while(line[0]=='c') /* kernel version > ~2.1.84 */
-          fgets(line,BUFFLEN,f); /* cpu or disk or page */
+       while(l && !(line[0]=='i' && line[1]=='n' && line[2]=='t' && line[3]=='r'))
+          l=fgets(line,BUFFLEN,f); /* cpu or disk or page or swap or intr */
 
-       while(line[0]=='d') /* kernel version < ~2.4.0-test4 */
-          fgets(line,BUFFLEN,f); /* disk_* or page */
-
-       fgets(line,BUFFLEN,f); /* swap */
-
-       fgets(line,BUFFLEN,f); /* intr */
-
-       if(sscanf(line,"intr %lu%n",&current[0],&p)==1)
+       if(!l)
+          fprintf(stderr,"ProcMeter(%s): Unexpected 'intr' line in '/proc/stat'.\n"
+                         "    expected: 'intr ...'\n"
+                         "    found:    EOF",__FILE__);
+       else if(sscanf(line,"intr %lu%n",&intr,&p)==1)
          {
           for(i=0;i<N_INTR;i++)
-             if(sscanf(line+p,"%lu%n",&current[i+1],&pp)==1)
+             if(sscanf(line+p,"%lu%n",&intr,&pp)==1)
                {
                 char *type="unknown";
                 FILE *f2;
@@ -172,10 +172,15 @@ ProcMeterOutput **Initialise(char *options)
           for(i=0;i<nintr;i++)
              outputs[n++]=&intr_outputs[i];
 
+          for(i=0;i<N_INTR+1;i++)
+             current[i]=previous[i]=0;
+
           outputs[n]=NULL;
          }
        else
-          fprintf(stderr,"ProcMeter(%s): Unexpected 'intr' line in '/proc/stat'.\n",__FILE__);
+          fprintf(stderr,"ProcMeter(%s): Unexpected 'intr' line in '/proc/stat'.\n"
+                         "    expected: 'intr %%lu ...'\n"
+                         "    found:    %s",__FILE__,line);
       }
 
     fclose(f);
@@ -205,7 +210,7 @@ int Update(time_t now,ProcMeterOutput *output)
  if(now!=last)
    {
     FILE *f;
-    char line[BUFFLEN+1];
+    char line[BUFFLEN+1],*l;
     long *temp;
     int p,pp;
 
@@ -217,17 +222,9 @@ int Update(time_t now,ProcMeterOutput *output)
     if(!f)
        return(-1);
 
-    fgets(line,BUFFLEN,f); /* cpu */
-
-    while(line[0]=='c') /* kernel version > ~2.1.84 */
-       fgets(line,BUFFLEN,f); /* cpu or disk or page */
-
-    while(line[0]=='d') /* kernel version < ~2.4.0-test4 */
-       fgets(line,BUFFLEN,f); /* disk_* or page */
-
-    fgets(line,BUFFLEN,f); /* swap */
-
-    fgets(line,BUFFLEN,f); /* intr */
+    while((l=fgets(line,BUFFLEN,f))) /* cpu or disk or page or swap or intr */
+       if(line[0]=='i' && line[1]=='n' && line[2]=='t' && line[3]=='r')
+          break;
 
     sscanf(line,"intr %lu%n",&current[0],&p);
     for(i=0;i<nintr;i++)
